from optparse import OptionParser
import sys
from .debug import simple as debug_simple

opt_configs = [
{
    "short": "p",
    "long": "projectname",
    "dest": "projectname",
    "help": "the project name"
},
    {
        "short": "c",
        "long": "cfg",
        "dest": "cfg",
        "help": "the banyan configuration file name",
        "default": "banyan"
    },
    {
        "short": "u",
        "long": "giturl",
        "dest": "giturl",
        "help": "git url for the project",
        "default": None
    },
    {
        "short": "t",
        "long": "tag",
        "dest": "tag",
        "help": "tag of the project",
        "default": "v1.0.0"
    }
]
def get_options(sys_args = None):

    def configure_parser(parser):
        def add_opt(opt):
            parser.add_option("-"+opt["short"], \
                              "--" + opt["long"], \
                              dest=opt["dest"] if "dest" in opt else None, \
                              help=opt["help"] if "help" in opt else None, \
                              default = opt["default"] if "default" in opt else None)

        [add_opt(opt) for opt in opt_configs]
        
        (options, args) = parser.parse_args(debug_simple(sys_args or sys.argv, "original args"))

        class dumy:
            def __init__(self):
                self.projectname = options.projectname
                self.cfg = options.cfg
                self.command = args[1]
                self.giturl = options.giturl
                self.tag = options.tag
                
        return dumy()

    return configure_parser(OptionParser())



import os


def init_deploy(root_folder, project_name, config_name, build_handler, run_handler, error_handler):
    '''
build_handler: (configuration_folder, banayan_configuration_name)，if the configuration_folder has not already created, build_handler will be called.
run_handler: (configuration_folder), if the configuration_folder has already been created, it will call this handler
error_handler: (msg), raise error message friendly.
'''
    def banyan_config_path():
        return os.path.join(root_folder, project_name, "deploy", "banyan.cfg" if config_name == None else config_name + ".cfg")

    def config_path():
        return os.path.join(root_folder, project_name, "deploy", ".banyan" if config_name == None else "." + config_name)
    
    def check_structure():
        def check(path):
            return "%s is not existing" % path if not os.path.exists(path) else None
        return check(banyan_config_path())









    if check_structure():
        error_handler(check_structure())

    if os.path.exists(config_path()):
        run_handler(config_path())

    if not os.path.exists(config_path()):
        build_handler(config_path(), banyan_config_path())


def run_deploy(configuration_path):
    '''configuration_path: it is the folder path which contains the main.sh to launch the deploy script'''
    import subprocess
    
    def run(current_path):
        try:
            os.chdir(configuration_path)
            p = subprocess.Popen("bash main.sh", shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

            while p.poll() is None:
                line = p.stdout.readline()
                if line:
                    print(line.strip())
        finally:
            os.chdir(current_path)

    run(os.getcwd())



import os
import shutil
from .assert_util import not_none as assert_not_none

root_folder = lambda:"./.tests"

def init_folder():
    if os.path.exists(root_folder()):
        remove_folder()
        
    os.makedirs(root_folder())

def remove_folder():
    shutil.rmtree(root_folder())
    
def create_folder(folder_name, parent_folder = None):
    def create(folder_path):
        if not os.path.exists(folder_path):
            os.makedirs(folder_path)
        return folder_path

    return create(os.path.join(parent_folder or root_folder(), folder_name))
                
def create_file(file_name, parent_folder, content):
    with open(os.path.join(parent_folder, file_name), 'w') as fileobj:
        fileobj.write(content)

def do_nothing():
    pass

def get_project_path(config_path):
    import re
    def match(pattern):
        return re.match(pattern, config_path)[0] if re.match(pattern, config_path) else None
    return os.path.relpath(assert_not_none(match('''.*(?=/deploy/\w+\.cfg)'''), "cfg path is invalid. Please make sure the it is under project folder"))
    



import demjson
import os


dependencies = lambda:"dependencies"

def load_config(config_path, handler):

    def handle(dep_data):
        def handle_config(config_data):
            if dependencies() in config_data:
                [handle(dep) for dep in config_data[dependencies()]]
                
        handle_config(handler(dep_data))

    def handle_from_root(config_data):
        if dependencies() in config_data:
            [handle(dep) for dep in config_data["dependencies"]]

    handle_from_root(demjson.decode_file(config_path))
     



import os
from codegenhelper import debug
import subprocess
import re

def get_project_name_from_url(giturl):
    return os.path.basename(giturl)

def get_tag(giturl, tag, location = os.getcwd()):
    def get_tag_func(project_name):
        debug("start to get {tag} of {project_name}".format(tag=tag, project_name=project_name))
        #
        def load(temp_folder):
            subprocess.call('mkdir {temp}'.format(temp=temp_folder), \
                            shell=True, \
                            cwd=location)
            subprocess.call(debug('wget {giturl}/archive/{tag}.tar.gz && tar vzxf {tag}.tar.gz -C {temp} && rm {tag}.tar.gz'.format(giturl=giturl, tag=tag, project_name=project_name, temp=temp_folder)), \
                            shell=True, \
                            cwd=location)
            subprocess.call('ls |xargs -I {{}} mv {{}} {project_name}'.format(project_name=project_name), \
                            shell=True, cwd=os.path.join(location, '.temp'))
            subprocess.call('mv {temp}/{project_name} ./ && rm {temp} -rf'.format(project_name=project_name, temp=temp_folder),\
                            shell=True, cwd=location)
        load('.temp')

    get_tag_func(get_project_name_from_url(giturl))
    
def get_code(giturl, ssh_key = None, location = os.getcwd()):
    debug("start to clone code of %s" % giturl)

    def is_ssh():
        return giturl.startswith('git')
    
    subprocess.call(debug('ssh -i {key_file} git clone {giturl}'.format(key_file=ssh_key, giturl=giturl),'command'), \
                    shell=True, \
                    cwd=location) if is_ssh() else \
                    subprocess.call(debug('git clone {giturl}'.format(giturl=giturl), 'command'), \
                                    shell=True, \
                                    cwd=location)
    
    



from .defaults import write as write_defaults, override_values, all as all_defaults
from .roles import build as roles_build, link as roles_link, load as roles_load, link_src_to_deploy
from .debug import simple as debug_simple
from .banyan_opt import get_options
import sys
import os
import yaml
from .tools.file import get_file_name, get_file_only_name, put_file
from .tools.folder import put_folder
import demjson
from .deploy import init_deploy, run_deploy
import logging
logging.basicConfig(stream = sys.stdout, level = 'DEBUG')

def run(options):
    
    def build_deploy_script(target_folder, cfg_file_path): #yml_file_path#
        yml_file_folder = lambda: put_folder(os.path.abspath(target_folder))
        yml_file_path = lambda: os.path.join(yml_file_folder(), "main.yml")
        host_file = lambda: os.path.join( \
            yml_file_folder(), \
            debug_simple(get_file_only_name(yml_file_path()), "host file name") + ".host")
        bash_file = lambda: os.path.join( \
                                          yml_file_folder(),
                                          get_file_only_name(yml_file_path()) + ".sh")

                                           
        def build(file, roles_data):
            # write data to file
            file.write(roles_build(roles_data))
            # build link for role folders
            roles_link(roles_data, \
                       debug_simple(yml_file_folder(), "link_root") \
            )
            # link src folder to deploy/roles/main/files/src for deployment
            link_src_to_deploy(roles_data)
            # build inventory files on roles
            write_defaults(host_file(), \
                           override_values( \
                                                     demjson.decode_file(cfg_file_path), \
                                                     all_defaults(yml_file_folder(), \
                                                                  [role["name"] for role in roles_data] \
                                                                  ) \
                                                     ) \
                           )
            # build the bash file
            open(bash_file(), "w") \
                .write("sudo ansible-playbook ./{yml_file} -i ./{host_file}" \
                       .format(yml_file = get_file_name(put_file(yml_file_path())), \
                               host_file = get_file_name(host_file()) \
                       ) \
                )

            
        build(open(put_file(yml_file_path()), 'w'), \
              roles_load(cfg_file_path) \
        )

        run_deploy(target_folder)
        
    def error_handler(msg):
        print(msg)

    def run_deploy_script(configuration_path):
        run_deploy(configuration_path)
        
    if options.command == "deploy":
        init_deploy(os.getcwd(), options.projectname, options.cfg, \
                build_deploy_script, \
                run_deploy_script, \
                error_handler)
    elif options.command == "init":
        from .init_code_with_tag import init_tag_for_project
        init_tag_for_project(options.giturl, options.tag, options.cfg, os.getcwd())
    else:
        print('please set the valid command: deploy, init')
        
def main():
    run(get_options())
